/* elasticbullets.js

Elastic band bullets
Written by Pintr Gbor
Feuerwehrgasse 1/2/9
Purbach am Neusiedlersee
A-7083, Austria
Tel: +43 681 20821067, +36 20 4331532
Email: pinter.gabor@gmx.at
Web: http://www.pic18.eu

Revisions:
  V1.0  ????-??-??  Original:  Philip Winston (pwinston@yahoo.com)
                    Web Site:  http://members.xoom.com/ebullets
                    'Scrolling Fix' by Troels Jakobsen <tjak@get2net.dk>
		    Downloaded from: The JavaScript Source!! http://javascript.internet.com
  V2.0  2001-10-15  Better documentation, more parameters
  V2.1  2001-12-08  NS6.1
  V2.2  2011-11-26  IE7, IE8
  V3.0  2014-03-14  Object oriented



This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 3
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

http://www.gnu.org/copyleft/gpl.html

For commercial license, and for other professional
JavaScript and Java components please contact the author.





Usage:
  1. Extract the contents of the zip file into the same directory where your page is.
  2. Include this file from the head of your page.
  3. Define parameters or accept the defaults.

This script requires Internet Explorer 5+, Nescape Navigator 6+!,
or any version of Firefox, Opera, Chrome or Safari.



1. Extract the contents of the zip file into the same directory where your page is.


2. Include elesticbullets.js from the head of your page
Insert the following line into the head of your page:
  <script type="text/javascript" src="elasticbullets.js"></script>


3. Define parameters
You can accept the defaults or assign new values to these variables:

Number of dots.
  ebb.numdots= 11;

Background color. (Can be transparent ='rgba(0,0,0,0)')
  ebb.background= 'blue';

The image that follows the cursor.
  ebb.img= "bullet.gif";

Use this instead of ebimg if you want something else to follow the cursor.
  ebb.html= null;

Size of the image or the html object.
  ebb.width= 11; ebb.height= 11; ebb.radius= 0;

Length of the elastic band between two dots.
  ebb.seglength= 20;

Spring force constant.
  ebb.springk= 10;

Mass of a bullet.
  ebb.mass= 1;

Gravity constant.
  ebb.gravity= 50;

Resistance loss constant.
  ebb.resistance= 10;

Bounce factor.
  ebb.bounce= 0.75;

Define z order position of image.
  ebb.zindex= 5;


4. Change parameters after the page is loaded
You can change parameters after the page is loaded,
e.g. using user inputs from edit boxes,
but you have to call ebb.create() to activate those changes.



Example: http://www.pic18.eu/en/www/elasticbullets/

*/



//------------------------------------------------------------------------------
// IE compatibility
//------------------------------------------------------------------------------



if (typeof(window.innerWidth)=='number') {
	window.getInnerWidth= function() {
		return window.innerWidth;
	};
} else if (document.documentElement && (document.documentElement.clientWidth || document.documentElement.clientHeight)) {
	// IE 6+ in 'standards compliant mode'
	window.getInnerWidth= function() {
		return document.documentElement.clientWidth;
	};
} else if (document.body && (document.body.clientWidth || document.body.clientHeight)) {
	// IE 4 compatible
	window.getInnerWidth= function() {
		return document.body.clientWidth;
	};
}
if (typeof(window.innerHeight)=='number') {
	window.getInnerHeight= function() {
		return window.innerHeight;
	};
} else if (document.documentElement && (document.documentElement.clientWidth || document.documentElement.clientHeight)) {
	// IE 6+ in 'standards compliant mode'
	window.getInnerHeight= function() {
		return document.documentElement.clientHeight;
	};
} else if (document.body && (document.body.clientWidth || document.body.clientHeight)) {
	// IE 4 compatible
	window.getInnerHeight= function() {
		return document.body.clientHeight;
	};
}



//------------------------------------------------------------------------------
// Mouse capture
//------------------------------------------------------------------------------



// Mouse position
window.mouse= { clientX:0, clientY:0 };



// Capture mouse position
window.addEventListener('mousemove', function(e) {
	window.mouse.clientX= e.clientX;
	window.mouse.clientY= e.clientY;
});



//------------------------------------------------------------------------------
// ebb object
//------------------------------------------------------------------------------



// Create empty
var ebb= {};



// Add public parameters with default values
ebb.numdots= 10;
ebb.background= 'blue';
ebb.img= null;
ebb.html= null;
ebb.width= 10;
ebb.height= 10;
ebb.radius= 5;
ebb.seglength= 20;
ebb.springk= 10;
ebb.mass= 1;
ebb.gravity= 50;
ebb.resistance= 10;
ebb.bounce= 0.75;
ebb.zindex= 5;



// Add private variables
ebb.deltat=0.02;
ebb.dots= [];
ebb.timer= null;



// Create/recreate
ebb.create= function() {
	//-------------
	// Delete existing timer
	if (ebb.timer) {
		window.clearInterval(this.timer);
		this.timer= null;
	}
	//-------------
	// Delete existing dots
	while (0 < ebb.dots.length) {
		var d= ebb.dots.pop();
		if (d.obj) {
			document.body.removeChild(d.obj);
		}
	}
	//-------------
	// Create one dot
	function dot(x, y, obj) {
		this.x= x;
		this.y= y;
		this.dx= 0;
		this.dy= 0;
		this.obj= obj;
	}
	//-------------
	// Create dots
	var x= window.mouse.clientX;
	var y= window.mouse.clientY;
	// First dot is at the cursor and invisible
	ebb.dots[0]= new dot(x, y, null);
	// Other dots are visible
	for (var i= 1; i <= ebb.numdots; i++) {
		y+= ebb.seglength;
		var obj= document.createElement('div');
		obj.style.position= 'fixed';
		obj.style.left= x+'px';
		obj.style.top= y+'px';
		obj.style.overflow= 'hidden';
		obj.style.lineHeight= ebb.height+'px';
		obj.style.height= ebb.height+'px';
		obj.style.width= ebb.width+'px';
		obj.style.borderRadius= ebb.radius+'px';
		if (ebb.html) {
			obj.innerHTML= ebb.html;
		} else if (ebb.img) {
			obj.innerHTML=
				"<img src='"+ebb.img+
				"' height='"+ebb.height+
				"' width='"+ebb.width+
				"' border='0' />";
		} else {
			obj.innerHTML= '';
		}
		obj.style.background= ebb.background;
		obj.style.zIndex= ebb.zindex;
		obj.style.pointerEvents= 'none';
		document.body.appendChild(obj);
		ebb.dots[i]= new dot(x, y, obj);
	}
	//-------------
	// Animate
	ebb.timer= window.setInterval(function() {
		//-----
		// Move object o
		function move(o, x, y) {
			if (o && o.style) {
				o.style.left= x+"px";
				o.style.top= y+"px";
			}
		}
		//-----
		// Vector object
		function vector(x, y) {
			this.x= x;
			this.y= y;
		}
		//-----
		// Add force in x and y direction to spring for ebdot[i] on ebdot[j]
		function force(i, j, spring) {
			var dx= ebb.dots[i].x-ebb.dots[j].x;
			var dy= ebb.dots[i].y-ebb.dots[j].y;
			var len= Math.sqrt(dx*dx+dy*dy);
			if (len > ebb.seglength) {
				var springforce= ebb.springk*(len-ebb.seglength);
				spring.x+= (dx/len)*springforce;
				spring.y+= (dy/len)*springforce;
			}
		}
		//-----
		// Window size
		var winwidth= window.getInnerWidth();
		var winheight= window.getInnerHeight();
		//-----
		// Calculate new positions
		ebb.dots[0].x= window.mouse.clientX;
		ebb.dots[0].y= window.mouse.clientY;
		for (var i= 1; i < ebb.dots.length; i++) {
			// Create spring
			var spring= new vector(0, 0);
			force(i-1, i, spring);
			if (i < ebb.dots.length-1) {
				force(i+1, i, spring);
			}
			// Resistance
			var resist= new vector(
				-ebb.dots[i].dx*ebb.resistance,
				-ebb.dots[i].dy*ebb.resistance);
			// Acceleration
			var accel= new vector(
				(spring.x+resist.x)/ebb.mass,
				(spring.y+resist.y)/ebb.mass+ebb.gravity);
			// New position, and new change of position
			ebb.dots[i].dx+= (ebb.deltat*accel.x);
			ebb.dots[i].dy+= (ebb.deltat*accel.y);
			ebb.dots[i].x+= ebb.dots[i].dx;
			ebb.dots[i].y+= ebb.dots[i].dy;
			// Bounce
			if (ebb.dots[i].y >= winheight-ebb.height/2-1) {
				if (ebb.dots[i].dy > 0) {
					ebb.dots[i].dy= -ebb.bounce*ebb.dots[i].dy;
				}
				ebb.dots[i].y= winheight-ebb.height/2-1;
			}
			if (ebb.dots[i].y < ebb.height/2) {
				if (ebb.dots[i].dy < 0) {
					ebb.dots[i].dy= -ebb.bounce*ebb.dots[i].dy;
				}
				ebb.dots[i].y= ebb.height/2;
			}
			if (ebb.dots[i].x >= winwidth-ebb.width/2) {
				if (ebb.dots[i].dx > 0) {
					ebb.dots[i].dx= -ebb.bounce*ebb.dots[i].dx;
				}
				ebb.dots[i].x= winwidth-ebb.width/2-1;
			}
			if (ebb.dots[i].x < ebb.width/2) {
				if (ebb.dots[i].dx < 0) {
					ebb.dots[i].dx= -ebb.bounce*ebb.dots[i].dx;
				}
				ebb.dots[i].x= ebb.width/2;
			}
			// Move to new position
			move(ebb.dots[i].obj,
				ebb.dots[i].x-ebb.width/2,
				ebb.dots[i].y-ebb.height/2);
		}
	}, 1000*ebb.deltat);
};



// Create after the page is loaded
window.onload= function() { ebb.create(); };


